BossPanel_Quests = BossPanelPlugin:new({
	name          = BossPanel_QuestsLocals.NAME,
	description   = BossPanel_QuestsLocals.DESCRIPTION,
	version       = "0.1.4",
	releaseDate   = "03-26-2006",
	aceCompatible = 103,
	bpCompatible  = 900,
	author        = "Kemayo",
	email         = "kemayo@gmail.com",
	website       = "http://kemayo.wowinterface.com/",
	category      = "quests",
	db            = AceDatabase:new("BossPanel_QuestsDB"),
	defaults      = {
		showLevels = TRUE,
		showDifficulty = TRUE,
		showArea = FALSE,
		showCurrentAreaOnly = FALSE,
		showCurrentAreaDescriptionOnly = FALSE,
		showDescription = TRUE,
		showCompletedObjectives = FALSE,
		showInItemTooltip = TRUE,
		colorObjectives = TRUE,
	},
	cmd           = AceChatCmd:new(BossPanel_QuestsLocals.COMMANDS, BossPanel_QuestsLocals.CMD_OPTIONS),
	loc           = BossPanel_QuestsLocals,
	hasIcon       = TRUE,
	canHideText   = TRUE,
	overrideTooltip = TRUE,
	
	-- Methods
	ShowTooltip = function(self)
		self:OpenChildFrame(BossPanel_QuestsFrame)
		self.idleTime = 0
	end,
	
	Frame_OnUpdate = function(self)
		if not self.enteredFrame then
			self.idleTime = self.idleTime + arg1
			if self.idleTime >= 2 then
				BossPanel_QuestsFrame:Hide()
			end
		end
	end,
	
	Frame_OnEnter = function(self)
		self.enteredFrame = TRUE
	end,
	
	Frame_OnLeave = function(self)
		self.enteredFrame = FALSE
	end,
	
	Quest_Click = function(self)
		self.idleTime = 0
		if this:GetID()>20 then
			self.quests[this:GetID()-20].hide = not self.quests[this:GetID()-20].hide
			self:UpdateTooltip()
		else
			local id = this:GetID()
			local questIndex
			for i, zone in self.quests do
				if not zone.hide then
					for ii, quest in zone.quests do
						id = id-1
						if id==0 then
							questIndex = quest.id
						end
					end
				end
			end
			
			if IsShiftKeyDown() then			
				-- Shift-click toggles quest-watch on this quest.
				if ( IsQuestWatched(questIndex) ) then
					RemoveQuestWatch(questIndex);
					QuestWatch_Update();
				else
					-- Set error if no objectives
					if ( GetNumQuestLeaderBoards(questIndex) == 0 ) then
						UIErrorsFrame:AddMessage(QUEST_WATCH_NO_OBJECTIVES, 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
						return;
					end
					-- Set an error message if trying to show too many quests
					if ( GetNumQuestWatches() >= MAX_WATCHABLE_QUESTS ) then
						UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, MAX_WATCHABLE_QUESTS), 1.0, 0.1, 0.1, 1.0, UIERRORS_HOLD_TIME);
						return;
					end
					AddQuestWatch(questIndex);
					QuestWatch_Update();
				end
			else
				if QuestLogFrame:IsVisible() then
					if self.lastIndex == questIndex then
						HideUIPanel(QuestLogFrame)
					end
				else
					ShowUIPanel(QuestLogFrame)
				end		
				QuestLog_SetSelection(questIndex)
				self.lastIndex = questIndex
			end
		end		
	end,
	
	IsShowingLevels = function(self)
		return self.data.showLevels
	end,
	
	ToggleShowingLevels = function(self, silent)
		self.data.showLevels = not self.data.showLevels
		return self.data.showLevels
	end,
	
	IsShowingDifficulty = function(self)
		return self.data.showDifficulty
	end,
	
	ToggleShowingDifficulty = function(self, silent)
		self.data.showDifficulty = not self.data.showDifficulty
		return self.data.showDifficulty
	end,
	
	IsShowingArea = function(self)
		return self.data.showArea
	end,
	
	ToggleShowingArea = function(self, silent)
		self.data.showArea = not self.data.showArea
		return self.data.showArea
	end,
	
	IsShowingCurrentAreaOnly = function(self)
		return self.data.showCurrentAreaOnly
	end,
	
	ToggleShowingCurrentAreaOnly = function(self, silent)
		self.data.showCurrentAreaOnly = not self.data.showCurrentAreaOnly
		return self.data.showCurrentAreaOnly
	end,
	
	IsShowingCurrentAreaDescriptionOnly = function(self)
		return self.data.showCurrentAreaDescriptionOnly
	end,
	
	ToggleShowingCurrentAreaDescriptionOnly = function(self, silent)
		self.data.showCurrentAreaDescriptionOnly = not self.data.showCurrentAreaDescriptionOnly
		return self.data.showCurrentAreaDescriptionOnly
	end,
	
	IsShowingDescription = function(self)
		return self.data.showDescription
	end,
	
	ToggleShowingDescription = function(self, silent)
		self.data.showDescription = not self.data.showDescription
		return self.data.showDescription
	end,
	
	IsColoringObjectives = function(self)
		return self.data.colorObjectives
	end,
	
	ToggleColoringObjectives = function(self, silent)
		self.data.colorObjectives = not self.data.colorObjectives
		return self.data.colorObjectives
	end,
	
	IsShowingCompletedObjectives = function(self)
		return self.data.showCompletedObjectives
	end,
	
	ToggleShowingCompletedObjectives = function(self, silent)
		self.data.showCompletedObjectives = not self.data.showCompletedObjectives
		return self.data.showCompletedObjectives
	end,
	
	IsShowingInItemTooltip = function(self)
		return self.data.showInItemTooltip
	end,
	
	ToggleShowingInItemTooltip = function(self, silent)
		self.data.showInItemTooltip = not self.data.showInItemTooltip
		return self.data.showDescription
	end,
	
	Initialize = function(self)
		self.numTooltipItems = 0
		self.numQuests = 0
		self.quests = {}
		self.items = {}
		self.mobs = {}
		
		BossPanel_QuestsFrameText:SetText(BossPanel_QuestsLocals.TEXT_QUESTS)
		BossPanel_Quest1:SetPoint("TOPLEFT", 3, -30)
		local prev, button = BossPanel_Quest1
		BossPanel_Quest1Left:SetWidth(275)
		for i=2, 30 do
			button = getglobal("BossPanel_Quest"..i)
			button:SetPoint("TOPLEFT", prev, "BOTTOMLEFT")
			getglobal("BossPanel_Quest"..i.."HLeft"):SetWidth(275)
			getglobal("BossPanel_Quest"..i.."Left"):SetWidth(260)
			prev = button
		end
	end,
	
	Enable = function(self)
		--self:Update()
		self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
		self:RegisterEvent("PLAYER_LEAVING_WORLD", "OnPlayerLeavingWorld")		
		
		--The next two are for displaying quest objectives in mouseover tooltips.
		self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "OnMouseOverUnit")
		--self:RegisterEvent("TOOLTIP_ANCHOR_DEFAULT", "OnItemTooltip")
		-- This should hook into tooltips:
		self:Hook("ContainerFrameItemButton_OnEnter", "OnItemTooltip")
	end,
	
	OnPlayerEnteringWorld = function(self)
		self:RegisterEvent("QUEST_LOG_UPDATE", "Update")
		self:RegisterEvent("PLAYER _LEVEL_UP", "Update")
	end,
	
	OnPlayerLeavingWorld = function(self)
		self:UnregisterEvent("QUEST_LOG_UPDATE")
		self:UnregisterEvent("PLAYER _LEVEL_UP")
	end,
	
	OnMouseOverUnit = function(self)
		-- Make sure that this is wanted, and that the tooltip is actually read/writeable.
		if self:IsShowingInItemTooltip() and self:CanMessWithGameTooltip() then
			local thisMob = getglobal('GameTooltipTextLeft1'):GetText()
			for _,mob in self.mobs do
				if string.find(thisMob, mob.mob) then
					self:AddToGameTooltipAndFixHeight(mob.quest..": "..mob.needed, mob.color)
				end
			end
		end
	end,
	
	OnItemTooltip = function(self)
		self.Hooks.ContainerFrameItemButton_OnEnter.orig()
		if self:IsShowingInItemTooltip() and self:CanMessWithGameTooltip() then
			local thisItem = getglobal('GameTooltipTextLeft1'):GetText()
			for _,item in self.items do
				if string.find(thisItem, item.item) then
					self:AddToGameTooltipAndFixHeight(item.quest..": "..item.needed, item.color)
				end
			end
		end
	end,
	
	CanMessWithGameTooltip = function(self)
		if GameTooltip and not getglobal('GameTooltipTextLeft1'):IsVisible() and getglobal('GameTooltipTextLeft1'):GetText() ~= nil then
			return TRUE
		else
			return FALSE
		end
	end,
	
	AddToGameTooltipAndFixHeight = function(self,text, color)
		GameTooltip:AddLine(text, color.r, color.g, color.b)
		
		-- Fix up the width of the tooltip
		length = getglobal(GameTooltip:GetName() .. "TextLeft" .. GameTooltip:NumLines()):GetStringWidth()
		-- Space for right-border:
		length = length + 22
		
		GameTooltip:SetHeight(GameTooltip:GetHeight() + 14)
		if length > GameTooltip:GetWidth() then
			GameTooltip:SetWidth(length)
		end
	end,
	
	Report = function(self)
		self.cmd:report({
			{text=self.loc.ARGUMENT_LEVELS, val=(self:IsShowingLevels() and 1 or 0), map=BossPanelLocals.MAP_ONOFF},
			{text=self.loc.ARGUMENT_DIFFICULTY, val=(self:IsShowingDifficulty() and 1 or 0), map=BossPanelLocals.MAP_ONOFF},
			{text=self.loc.ARGUMENT_AREA, val=(self:IsShowingArea() and 1 or 0), map=BossPanelLocals.MAP_ONOFF},
		})
	end,
	
	MenuSettings = function(self)
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_LEVELS,
			func = function()
				self:ToggleShowingLevels()
			end,
			checked = self:IsShowingLevels(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_DIFFICULTY,
			func = function()
				self:ToggleShowingDifficulty()
			end,
			checked = self:IsShowingDifficulty(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_AREA,
			func = function()
				self:ToggleShowingArea()
			end,
			checked = self:IsShowingArea(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_CURRENT_AREA_ONLY,
			func = function()
				self:ToggleShowingCurrentAreaOnly()
			end,
			checked = self:IsShowingCurrentAreaOnly(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_DESCRIPTION,
			func = function()
				self:ToggleShowingDescription()
			end,
			checked = self:IsShowingDescription(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_COLOR_OBJECTIVES,
			func = function()
				self:ToggleColoringObjectives()
			end,
			checked = self:IsColoringObjectives(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_COMPLETED_OBJECTIVES,
			func = function()
				self:ToggleShowingCompletedObjectives()
			end,
			checked = self:IsShowingCompletedObjectives(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_CURRENT_AREA_DESCRIPTION_ONLY,
			func = function()
				self:ToggleShowingCurrentAreaDescriptionOnly()
			end,
			checked = self:IsShowingCurrentAreaDescriptionOnly(),
			keepShownOnClick = TRUE,
		})
		
		UIDropDownMenu_AddButton({
			text = self.loc.MENU_SHOW_IN_ITEM_TOOLTIP,
			func = function()
				self:ToggleShowingInItemTooltip()
			end,
			checked = self:IsShowingInItemTooltip(),
			keepShownOnClick = TRUE,
		})
	end,
	
	UpdateData = function(self)
		--UpdateText and UpdateTooltip both need to know things about the questlog.  Better to do this once.
		local startingQuestLogSelection = GetQuestLogSelection()
		local numEntries, numQuests = GetNumQuestLogEntries()
		local quests = {}
		local items = {}
		local mobs = {}
		local zoneIndex = 0
		local quest
		
		for quest = 1, numEntries, 1 do
			SelectQuestLogEntry(quest)
			local strQuestLogTitleText, strQuestLevel, strQuestTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(quest)
			local strQuestDescription, strQuestObjectives = GetQuestLogQuestText()
			local q = {}
			local thisQuest = "  "
			local strQuestLevelTag = ""
			
			if strQuestTag == self.loc.ELITE then
				strQuestLevelTag = "+"
			elseif strQuestTag == self.loc.DUNGEON then
				strQuestLevelTag = "D"
			elseif strQuestTag == self.loc.PVP then
				strQuestLevelTag = "P"
			elseif strQuestTag == self.loc.RAID then
				strQuestTag = "R"
			end
			
			if isHeader then
				zoneIndex = zoneIndex + 1
				quests[zoneIndex] = { ["zone"]=strQuestLogTitleText, ["quests"]={} }
			else
				if isComplete == 1 then
					isComplete = format("(%s)",self.loc.QUEST_DONE)
				elseif isComplete == -1 then
					isComplete = format("(%s)",self.loc.QUEST_FAILED)
				else
					isComplete = ""
				end
				
				q.title = strQuestLogTitleText
				q.level = strQuestLevel
				q.tag = strQuestLevelTag
				q.complete = isComplete
				q.objective = strQuestObjectives
				
				local lb = {}
				if (GetNumQuestLeaderBoards() > 0) then
					for ii=1, GetNumQuestLeaderBoards(), 1 do
						local desc, qtype, done = GetQuestLogLeaderBoard(ii)
						local forcolor=desc
						
						local _, _, itemName, numItems, numNeeded = string.find(desc, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$")
						if qtype == 'item' then
							desc = itemName
							table.insert(items, {quest = strQuestLogTitleText, item = itemName, needed = numItems.."/"..numNeeded, color=BossPanelUtils.GetThresholdColor(numItems / numNeeded)})
						elseif qtype == 'monster' then
							_,_,mobName = string.find(itemName, "(.*) %a+%s*$")
							desc = mobName
							table.insert(mobs, {quest = strQuestLogTitleText, mob = mobName, needed = numItems.."/"..numNeeded, color=BossPanelUtils.GetThresholdColor(numItems / numNeeded)})
						end
						
						if done then
							done = format("(%s)",self.loc.QUEST_DONE)
						elseif numItems and numNeeded then
							done = string.format("%s/%s",numItems,numNeeded)
						else
							done = ""
						end
						--if not forcolor then forcolor = "nil" end
						--if not desc then message(string.format("Debug: %s: lb%d (%s - %s)",q['title'],ii,qtype, forcolor))
						else
							lb[ii] = {description=desc, questtype=qtype, done=done, forcolor=forcolor}
						end
					end
				end
				q.leaderboard = lb
				q.id = quest
				
				table.insert(quests[zoneIndex].quests,q)
			end
		end
		
		self.quests = quests
		self.items = items
		self.mobs = mobs
		self.numQuests = numQuests
		
		SelectQuestLogEntry(startingQuestLogSelection)
	end,
	
	UpdateText = function(self)
		local maxQuests = 20
		local color = BossPanelUtils.GetThresholdHexColor((maxQuests - self.numQuests) / maxQuests)
		
		self:SetText(BossPanelUtils.Colorize(color,format("%d/%d", self.numQuests, maxQuests)))
	end,
	
	AddElement = function(self, Hleft, Hright, id, checked, left, right)
		self.numBut = self.numBut + 1
		local i = self.numBut
		local button = getglobal("BossPanel_Quest"..i)		
		if checked then
			getglobal("BossPanel_Quest"..i.."Check"):Show()
		else
			getglobal("BossPanel_Quest"..i.."Check"):Hide()
		end
		getglobal("BossPanel_Quest"..i.."HLeft"):SetText(Hleft)
		getglobal("BossPanel_Quest"..i.."HRight"):SetText(Hright or "")
		local height = getglobal("BossPanel_Quest"..i.."HLeft"):GetHeight()+2
		if left then
			getglobal("BossPanel_Quest"..i.."Left"):SetText(left)
			getglobal("BossPanel_Quest"..i.."Right"):SetText(right or "")
			getglobal("BossPanel_Quest"..i.."Left"):Show()
			getglobal("BossPanel_Quest"..i.."Right"):Show()
			height = height + getglobal("BossPanel_Quest"..i.."Left"):GetHeight()
		else
			getglobal("BossPanel_Quest"..i.."Left"):Hide()
			getglobal("BossPanel_Quest"..i.."Right"):Hide()
		end
		button:SetHeight(height)
		button:SetID(id)
		button:Show()
		return height
	end,
	
	UpdateTooltip = function(self)
		local currentZone = GetZoneText()
		local showArea = self:IsShowingArea()
		local showDescription = self:IsShowingDescription()
		local showCurrentAreaOnly = self:IsShowingCurrentAreaOnly()
		local showCurrentAreaDescriptionOnly = self:IsShowingCurrentAreaDescriptionOnly()
		local cat, qi, height = 0, 0, 40
		self.numBut = 0
		
		for _,zone in self.quests do
			if showArea and ((currentZone == zone.zone) or not showCurrentAreaOnly) then
				cat = cat + 1
				height = height + self:AddElement(BossPanelUtils.Colorize("bfffbf",zone['zone']), "", 20+cat, zone.Hide)
			end
			for _,quest in zone['quests'] do
				if not zone.hide and ((currentZone == zone.zone) or not showCurrentAreaOnly) then
					local thisQuest
					local questLevel = ""
					
					if self:IsShowingLevels() then
						questLevel = format("[%s%s] ", quest.level, quest.tag)
					end
					
					thisQuest = questLevel..quest.title
					
					local thisQuestColor = {r=1,g=1,b=1}
					if self:IsShowingDifficulty() then
						thisQuestColor = GetDifficultyColor(quest.level)
					end					
					
					local hexcolor = format("%02X%02X%02X", thisQuestColor.r * 255, thisQuestColor.g * 255, thisQuestColor.b * 255)
					local leftHeader = "  "..BossPanelUtils.Colorize(hexcolor, thisQuest)
					local rightHeader = quest.complete
					local left, right			
					
					if showDescription and ((currentZone == zone['zone']) or not showCurrentAreaDescriptionOnly) then
						if table.getn(quest['leaderboard']) > 0 then
							for _,goal in quest['leaderboard'] do
								if not (string.find(goal['done'], self.loc.QUEST_DONE) and not self:IsShowingCompletedObjectives()) then
									local color = {}
									if self:IsColoringObjectives() then
										color.r,color.g,color.b = self:GetGoalColor(goal['forcolor'])
									else 
										color = thisQuestColor
									end
									hexcolor = format("%02X%02X%02X", color.r * 255, color.g * 255, color.b * 255)
									left = (left and left.."\n" or "") .. BossPanelUtils.Colorize(hexcolor, goal.description)
									right = (right and right.."\n" or "") .. BossPanelUtils.Colorize(hexcolor, goal.done)
								end
							end
						elseif not (string.find(quest['complete'], self.loc.QUEST_DONE) and not self:IsShowingCompletedObjectives()) then
							left = BossPanelUtils.Colorize("7f7f7f", quest.objective)
						end
					end
					qi = qi + 1
					height = height + self:AddElement(leftHeader, rightHeader, qi, IsQuestWatched(quest.id), left, right)					
				end
			end
		end
		for i = self.numBut+1, 33 do
			getglobal("BossPanel_Quest"..i):Hide()
		end
		BossPanel_QuestsFrame:SetHeight(height)
	end,	
	
	OnClick = function(self)
		ToggleQuestLog()
	end,
	
	GetGoalColor = function(self, goalText)
		local _, _, itemName, numItems, numNeeded = string.find(goalText, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$");
		if numItems ~= nil then
			-- This quest involves items!			
			return BossPanelUtils.GetThresholdColorTrivial(numItems / numNeeded)
		else
			-- This quest doesn't involve items.  Let's leave it grey.  We're lazy.
			return 0.5,0.5,0.5
		end
	end,
})

BossPanel_Quests:RegisterForLoad()
